//		Copyright © 1997	Jens Kilian
//
//		This program is free software; you can redistribute it and/or modify
//		it under the terms of the GNU General Public License as published by
//		the Free Software Foundation; either version 1, or (at your option)
//		any later version.
//
//		This program is distributed in the hope that it will be useful,
//		but WITHOUT ANY WARRANTY; without even the implied warranty of
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
//		GNU General Public License for more details.
//
//		You should have received a copy of the GNU General Public License
//		along with this program; if not, write to the Free Software
//		Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//		$Log: RandomColor.cpp,v $
//		Revision 1.1  1999/01/10 09:48:19  jjk
//		Initial revision
//

//		Select a color of random hue, within a given range of lightness
//		and saturation.

#include <cstdlib>
#include <GraphicsDefs.h>

#include "RandomColor.h"

inline float
rgb(float q1, float q2, int hue)
{
	hue = (hue + 360) % 360;
	if (hue < 60) {
		return q1 + (q2-q1) * hue / 60.0;

	} else if (hue < 180) {
		return q2;

	} else if (hue < 240) {	
		return q1 + (q2-q1) * (240 - hue) / 60.0;
	
	} else {
		return q1;
	}
}

RandomColor::RandomColor(float minL, float maxL, float minS, float maxS,
								 int hue)
{
	if (hue < 0) {
		hue = rand();
	}
	hue %= 360;

	float lightness =		minL + (float)(rand() % 1000) / 1000.0 * (maxL-minL);
	float saturation =	minS + (float)(rand() % 1000) / 1000.0 * (maxS-minS);

	// HLS to RGB conversion.  Sheer magic, don't ask me :-)
	float p2 = (lightness <= 0.5)
		? lightness * (saturation + 1.0)
		: lightness + saturation - lightness*saturation;
	float p1 = 2.0*lightness - p2;
	
	if (saturation == 0.0) {
		mColor.red =		(int)(255.0 * lightness);
		mColor.green =		(int)(255.0 * lightness);
		mColor.blue =		(int)(255.0 * lightness);
	} else {
		mColor.red =		(int)(255.0 * rgb(p1, p2, hue + 120));
		mColor.green =		(int)(255.0 * rgb(p1, p2, hue));
		mColor.blue =		(int)(255.0 * rgb(p1, p2, hue - 120));
	}
}

static char RCSid[] = "$Id: RandomColor.cpp,v 1.1 1999/01/10 09:48:19 jjk Exp $";
