//		Copyright © 1997	Jens Kilian
//
//		This program is free software; you can redistribute it and/or modify
//		it under the terms of the GNU General Public License as published by
//		the Free Software Foundation; either version 1, or (at your option)
//		any later version.
//
//		This program is distributed in the hope that it will be useful,
//		but WITHOUT ANY WARRANTY; without even the implied warranty of
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
//		GNU General Public License for more details.
//
//		You should have received a copy of the GNU General Public License
//		along with this program; if not, write to the Free Software
//		Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//		$Log: RandomKnot.h,v $
//		Revision 1.1  1999/01/10 09:48:19  jjk
//		Initial revision
//

//		Base class for creating random knots.

#ifndef RANDOMKNOT_H
#define RANDOMKNOT_H

#include <GraphicsDefs.h>
#include <Point.h>
#include <View.h>

#include "KnotSection.h"
#include "KnotSectionCache.h"

class RandomKnot {
public:
	RandomKnot(
		BView *pView, int maxSections, int xRepeat = 1, int yRepeat = 1);
	virtual ~RandomKnot(void);

	bool IsValid(void) const;
	virtual bool Draw(void) = 0;

protected:
	// Find a position where the knot can be drawn.
	BPoint LocateKnot(void);
	
	// Return the size of the knot.
	BPoint Size(void) const;
	
	// Draw the knot at the given position.
	void DrawAt(BPoint where);
	
	// Data members.
	BView *mpView;								// The view on which we'll draw.
	const int mWidth;							// horizontal #sections
	const int mHeight;						// vertical #sections
	direction *mpSectionTypes;				// types of sections
	direction *mpSectionCorners;			// corner directions	

private:
	RandomKnot(const RandomKnot &orig);
	RandomKnot &operator =(RandomKnot &orig);
	
	// Data members.
	KnotSectionCache *mpCache;				// Section bitmap cache.
	const rgb_color mColor;					// Knot color.
};

inline bool
RandomKnot::IsValid(void) const
{
	return mpCache != 0;
}

inline BPoint
RandomKnot::Size(void) const
{
	if (IsValid()) {
		const float edgeLength = mpCache->EdgeLength();
		return BPoint(mWidth * edgeLength, mHeight * edgeLength);
	}
	return B_ORIGIN;
}

#endif
